#include "../LLVMParser.hpp"
#include "../Lexer.hpp"
#include <algorithm>
#include <cctype>
#include <cstdio>
#include <fstream>
#include <functional>
#include <gtest/gtest.h>
#include <locale>
#include <sstream>
#include <stdint.h>

std::string& ltrim(std::string& s)
{
    s.erase(s.begin(), std::find_if(s.begin(), s.end(),
                           std::not1(std::ptr_fun<int, int>(std::isspace))));
    return s;
}
std::string& rtrim(std::string& s)
{
    s.erase(std::find_if(s.rbegin(), s.rend(),
                std::not1(std::ptr_fun<int, int>(std::isspace)))
                .base(),
        s.end());
    return s;
}
std::string& trim(std::string& s)
{
    return ltrim(rtrim(s));
}

std::string compile_and_run(const std::string& input)
{
    {
        std::istringstream input_stream(input);
        Lexer lexer(input_stream, std::cout, std::cerr);
        std::ofstream output_stream("tests/LLVMParser.ll");
        LLVMParser parser(lexer, output_stream, std::cerr);
        parser.run();
    }
    system("lli tests/LLVMParser.ll > tests/LLVMParser.out");
    std::ifstream output_file("tests/LLVMParser.out");
    std::string output(
        (std::istreambuf_iterator<char>(output_file)), std::istreambuf_iterator<char>());
    return trim(output);
}

int32_t string_to_integer(const std::string& input)
{
    int32_t output;
    std::istringstream(input) >> output;
    return output;
}

double string_to_double(const std::string& input)
{
    double output;
    std::istringstream(input) >> output;
    return output;
}

TEST(LLVMParser, globalReturns)
{
    ASSERT_EQ(12345, string_to_integer(compile_and_run("return 12345;")));
    ASSERT_EQ(1.2345, string_to_double(compile_and_run("return 1.2345;")));
    ASSERT_EQ(0.12345, string_to_double(compile_and_run("return .12345;")));
}
